package mircoders.producer;

import mir.changetracker.ChangeTracker;
import mir.log.LoggerWrapper;
import mir.producer.AbstractProducerNode;
import mir.util.ParameterExpander;
import mir.util.StringRoutines;
import mircoders.global.MirGlobal;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class ChangeReportingProducerNode extends AbstractProducerNode {
  private String format;
  private String outputFile;
  private String basePath;
  private String exclusionList;

  public ChangeReportingProducerNode(String anOutputFile, String aFormat, String aBasePath, String anExclusionList) {
    format = aFormat;
    outputFile = anOutputFile;
    basePath = aBasePath;
    exclusionList = anExclusionList;
  }

  public void produce(Map aValueMap, String aVerb, LoggerWrapper aLogger) {
    try {
      File file = new File(ParameterExpander.expandExpression(aValueMap, outputFile));
      OutputStream stream = null;
      try {
        stream = new FileOutputStream(file, true);
      }
      catch (FileNotFoundException e) {
        aLogger.error("Can't write open report file", e);

        return;
      }

      try {
        PrintWriter writer = new PrintWriter(new OutputStreamWriter(stream));
        try {
          String expandedBasePath = ParameterExpander.expandExpression(aValueMap, basePath);

          List excludedPaths = null;
          if (exclusionList != null) {
            String expandedExclusionList = ParameterExpander.expandExpression(aValueMap, exclusionList);
            excludedPaths = StringRoutines.separateString(expandedExclusionList, ",");
          }

          Collection changes = MirGlobal.getChangeEngine().getTracker().flushChanges(expandedBasePath, excludedPaths);

          Iterator i = changes.iterator();
          while (i.hasNext()) {
            ChangeTracker.Change change = (ChangeTracker.Change) i.next();

            Map changeMap = new HashMap();

            // the absolute path of the change needs to be transformed into a path relative
            //    to base
            changeMap.put("path", change.getPath().substring(expandedBasePath.length()));
            changeMap.put("type", change.getType().getName());
            changeMap.put("date", change.getDate());

            aValueMap.put("change", changeMap);
            String line = ParameterExpander.expandExpression(aValueMap, format);
            writer.println(line);
          }
        }
        finally {
          try {
            writer.close();
          }
          catch (Throwable t) {
            aLogger.info("Can't close report file");
          }
        }
      }
      finally {
        try {
          stream.close();
        }
        catch (Throwable t) {
          aLogger.info("Can't close report file");
        }
      }
    }
    catch (Throwable t) {
      aLogger.error("Error while writing change report file", t);
    }
  }
}
