/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */

package mircoders.search;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;

import mir.entity.Entity;
import mir.entity.EntityBrowser;
import mir.storage.DatabaseFailure;
import mircoders.entity.EntityImages;
import mircoders.storage.DatabaseImages;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;

/**
 * Indexes whether or not an Entity has associated
 * images, and also stores urls of those images for 
 * retrieval in the search results
 */
public class ImagesSearchTerm extends SearchTerm{
  public ImagesSearchTerm() {
    super (null, "search_hasImages", "hasImages", "images", "images");
  }

  public void index(Document doc, Entity entity) throws DatabaseFailure{
    List extraTables = new ArrayList();
    extraTables.add("content_x_media cxm");

    Iterator images = new EntityBrowser(
					DatabaseImages.getInstance(), "i", extraTables,
					"cxm.content_id="+entity.getId()+"and cxm.media_id=i.id",
					"i.id desc", 30, -1, 0);


    if (images != null && images.hasNext()){
      doc.add(Field.Keyword(matchField,"y"));
      String imageURLString = "";
      int k=0;
      while(images.hasNext()){
        if (k != 0){
          imageURLString = imageURLString + ":";
        }
	      k++;
        imageURLString = imageURLString + ((EntityImages) images.next()).getFieldValue("icon_path");
      }
      doc.add(Field.UnIndexed("images",imageURLString));
    }
  }

  public String makeTerm(HttpServletRequest req){
    String wanted = req.getParameter(paramName);
    if (wanted != null && wanted.equals("y")){
      return matchField + ":y";
    }
		return null;
  }

  public void returnMeta(Map result,Document doc){
    String imageURLString=doc.get(dataField);
    List theImages = new ArrayList();
    if (imageURLString != null){
      StringTokenizer st = new StringTokenizer(imageURLString,":");
      while (st.hasMoreTokens()) {
        String imageURL=st.nextToken();
        theImages.add(imageURL);
      }
    }
    result.put(templateVariable,theImages);
  }


}


