/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */

package mircoders.storage;

import java.sql.Connection;
import java.sql.Statement;
import java.util.ArrayList;

import mir.entity.EntityList;
import mir.log.LoggerWrapper;
import mir.storage.Database;
import mir.storage.DatabaseFailure;
import mircoders.entity.EntityUploadedMedia;

/**
 * <b>implements abstract DB connection to the content_x_media SQL table
 *
 * @author RK, mir-coders group
 * @version $Id: DatabaseContentToMedia.java,v 1.19.2.10 2005/03/26 11:26:28 zapata Exp $
 *
 */

public class DatabaseContentToMedia extends Database {

  private static DatabaseContentToMedia instance;

  public synchronized static DatabaseContentToMedia getInstance() {
    if (instance == null) {
      instance = new DatabaseContentToMedia();
    }
    return instance;
  }

  private DatabaseContentToMedia() {
    super();

    logger = new LoggerWrapper("Database.ContentToMedia");
    mainTable = "content_x_media";
    entityClass = mir.entity.GenericEntity.class;
  }

  public void addMedia(String contentId, String mediaId) throws
      DatabaseFailure {
    if (contentId == null && mediaId == null) {
      return;
    }

    Connection con = null;
    Statement stmt = null;
    //now insert

    String sql = "insert into " + mainTable + " (content_id,media_id) values ("
        + contentId + "," + mediaId + ")";
    try {
      con = obtainConnection();
      // should be a preparedStatement because is faster
      stmt = con.createStatement();
      int rs = executeUpdate(stmt, sql);
    }
    catch (Exception e) {
      logger.error("-- add media failed -- insert");
      throw new DatabaseFailure("-- add media failed -- insert ", e);
    }
    finally {
      freeConnection(con, stmt);
    }
  }

  public void setMedia(String contentId, String mediaId) throws
      DatabaseFailure {
    if (contentId == null && mediaId == null) {
      return;
    }
    //first delete all row with content_id=contentId
    String sql = "delete from " + mainTable + " where content_id=" + contentId;

    Connection con = null;
    Statement stmt = null;
    try {
      con = obtainConnection();
      // should be a preparedStatement because is faster
      stmt = con.createStatement();
      int rs = executeUpdate(stmt, sql);
    }
    catch (Exception e) {
      logger.error("-- set media failed -- delete");
      throw new DatabaseFailure("-- set media failed -- delete ", e);
    }
    finally {
      freeConnection(con, stmt);
    }

    //now insert
    //first delete all row with content_id=contentId

    sql = "insert into " + mainTable + " (content_id,media_id) values ("
        + contentId + "," + mediaId + ")";
    try {
      con = obtainConnection();
      // should be a preparedStatement because is faster
      stmt = con.createStatement();
      int rs = executeUpdate(stmt, sql);
    }
    catch (Exception e) {
      logger.error("-- set media failed -- insert");
      throw new DatabaseFailure("-- set media failed -- insert ", e);
    }
    finally {
      freeConnection(con, stmt);
    }
  }

  public void deleteByContentId(String contentId) throws DatabaseFailure {
    if (contentId == null) {
      //theLog.printDebugInfo("-- delete topics failed -- no content id");
      return;
    }
    //delete all row with content_id=contentId
    String sql = "delete from " + mainTable + " where content_id=" + contentId;

    Connection con = null;
    Statement stmt = null;
    try {
      con = obtainConnection();
      // should be a preparedStatement because is faster
      stmt = con.createStatement();
      int rs = executeUpdate(stmt, sql);
    }
    catch (Exception e) {
      logger.error("-- delete by contentId failed  ");
      throw new DatabaseFailure(
          "-- delete by content id failed -- delete ", e);
    }
    finally {
      freeConnection(con, stmt);
    }
  }

  public void deleteByMediaId(String mediaId) throws DatabaseFailure {

    if (mediaId == null) {
      //theLog.printDebugInfo("-- delete topics failed -- no topic id");
      return;
    }
    //delete all row with content_id=contentId
    String sql = "delete from " + mainTable + " where media_id=" + mediaId;

    Connection con = null;
    Statement stmt = null;
    try {
      con = obtainConnection();
      // should be a preparedStatement because is faster
      stmt = con.createStatement();
      int rs = executeUpdate(stmt, sql);
      logger.debug("-- delete media success ");
    }
    catch (Exception e) {
      logger.error("-- delete media failed ");
      throw new DatabaseFailure("-- delete by media id failed -- ", e);
    }
    finally {
      freeConnection(con, stmt);
    }
  }

  public void delete(String contentId, String mediaId) throws
      DatabaseFailure {
    if (mediaId == null || contentId == null) {
      logger.debug("-- delete media failed -- missing parameter");
      return;
    }
    //delete all row with content_id=contentId and media_id=mediaId
    String sql = "delete from " + mainTable + " where media_id=" + mediaId +
        " and content_id= " + contentId;

    Connection con = null;
    Statement stmt = null;
    try {
      con = obtainConnection();
      // should be a preparedStatement because is faster
      stmt = con.createStatement();
      int rs = executeUpdate(stmt, sql);
      logger.debug("-- delete content_x_media success ");
    }
    catch (Exception e) {
      logger.error("-- delete content_x_media failed ");
      throw new DatabaseFailure("-- delete content_x_media failed -- ", e);
    }
    finally {
      freeConnection(con, stmt);
    }
  }

  public EntityList getContent(EntityUploadedMedia media) throws
      DatabaseFailure {

    EntityList returnList = null;
    if (media != null) {

      String id = media.getId();
      ArrayList extraTables = new ArrayList();
      extraTables.add(mainTable + " cxm");

      String mediaSelect = "cxm.content_id=c.id and cxm.media_id="+id;
      try {
        returnList = DatabaseContent.getInstance().selectByWhereClause("c",
          extraTables, mediaSelect, "c.id" );

      }
      catch (Exception e) {
        logger.error("-- get content failed");
        throw new DatabaseFailure("-- get content failed -- ", e);
      }
    }
    return returnList;
  }

  /**
   * Returns a EntityList with all content-objects having a relation to a media
   */

  public EntityList getContent() throws DatabaseFailure {

    EntityList returnList = null;

    ArrayList extraTables = new ArrayList();
    extraTables.add(mainTable + " cxm");

    String mediaSelect = "cxm.content_id=c.id";
    try {
      returnList = DatabaseContent.getInstance().selectByWhereClause("c",
        extraTables, mediaSelect, "c.webdb_lastchange desc" );

    }
    catch (Exception e) {
      logger.error("-- get content failed");
      throw new DatabaseFailure("-- get content failed -- ", e);
    }
    return returnList;
  }

}
